@@ -197,6 +197,7 @@ extern xfs_mount_t	*libxfs_mount (xfs_mount_t *, xfs_sb_t *,
 				dev_t, dev_t, dev_t, int);
 extern void	libxfs_mount_common (xfs_mount_t *, xfs_sb_t *);
 extern xfs_agnumber_t	libxfs_initialize_perag (xfs_mount_t *, xfs_agnumber_t);
+extern int	libxfs_initialize_perag_data (xfs_mount_t *, xfs_agnumber_t);
 extern void	libxfs_umount (xfs_mount_t *);
 extern int	libxfs_rtmount_init (xfs_mount_t *);
 extern void	libxfs_rtmount_destroy (xfs_mount_t *);
@@ -429,33 +430,33 @@ extern void	libxfs_dir_mount (xfs_mount_t *);
 extern void	libxfs_dir2_mount (xfs_mount_t *);
 extern int	libxfs_dir_init (xfs_trans_t *, xfs_inode_t *, xfs_inode_t *);
 extern int	libxfs_dir2_init (xfs_trans_t *, xfs_inode_t *, xfs_inode_t *);
-extern int	libxfs_dir_createname (xfs_trans_t *, xfs_inode_t *, char *,
+extern int	libxfs_dir_createname (xfs_trans_t *, xfs_inode_t *, uchar_t *,
 				int, xfs_ino_t, xfs_fsblock_t *,
 				xfs_bmap_free_t *, xfs_extlen_t);
-extern int	libxfs_dir2_createname (xfs_trans_t *, xfs_inode_t *, char *,
+extern int	libxfs_dir2_createname (xfs_trans_t *, xfs_inode_t *, uchar_t *,
 				int, xfs_ino_t, xfs_fsblock_t *,
 				xfs_bmap_free_t *, xfs_extlen_t);
 extern int	libxfs_dir_lookup (xfs_trans_t *, xfs_inode_t *,
-				char *, int, xfs_ino_t *);
+				uchar_t *, int, xfs_ino_t *);
 extern int	libxfs_dir2_lookup (xfs_trans_t *, xfs_inode_t *,
-				char *, int, xfs_ino_t *);
+				uchar_t *, int, xfs_ino_t *);
 extern int	libxfs_dir_replace (xfs_trans_t *, xfs_inode_t *,
-				char *, int, xfs_ino_t, xfs_fsblock_t *,
+				uchar_t *, int, xfs_ino_t, xfs_fsblock_t *,
 				xfs_bmap_free_t *, xfs_extlen_t);
 extern int	libxfs_dir2_replace (xfs_trans_t *, xfs_inode_t *,
-				char *, int, xfs_ino_t, xfs_fsblock_t *,
+				uchar_t *, int, xfs_ino_t, xfs_fsblock_t *,
 				xfs_bmap_free_t *, xfs_extlen_t);
 extern int	libxfs_dir_removename (xfs_trans_t *, xfs_inode_t *,
-				char *, int, xfs_ino_t, xfs_fsblock_t *,
+				uchar_t *, int, xfs_ino_t, xfs_fsblock_t *,
 				xfs_bmap_free_t *, xfs_extlen_t);
 extern int	libxfs_dir2_removename (xfs_trans_t *, xfs_inode_t *,
-				char *, int, xfs_ino_t, xfs_fsblock_t *,
+				uchar_t *, int, xfs_ino_t, xfs_fsblock_t *,
 				xfs_bmap_free_t *, xfs_extlen_t);
 extern int	libxfs_dir_bogus_removename (xfs_trans_t *, xfs_inode_t *,
-				char *, xfs_fsblock_t *, xfs_bmap_free_t *,
+				uchar_t *, xfs_fsblock_t *, xfs_bmap_free_t *,
 				xfs_extlen_t, xfs_dahash_t, int);
 extern int	libxfs_dir2_bogus_removename (xfs_trans_t *, xfs_inode_t *,
-				char *, xfs_fsblock_t *, xfs_bmap_free_t *,
+				uchar_t *, xfs_fsblock_t *, xfs_bmap_free_t *,
 				xfs_extlen_t, xfs_dahash_t, int);
 
 
